@@ -17,6 +17,7 @@ import (
 	"github.com/openshift/api/oauth"
 	operatorv1 "github.com/openshift/api/operator"
 	"github.com/openshift/console-operator/pkg/api"
+	"github.com/openshift/console-operator/pkg/console/controllers/clidownloads"
 	"github.com/openshift/console-operator/pkg/console/operatorclient"
 	"github.com/openshift/library-go/pkg/controller/controllercmd"
 	"github.com/openshift/library-go/pkg/operator/management"
@@ -39,7 +40,13 @@ import (
 	routesclient "github.com/openshift/client-go/route/clientset/versioned"
 	routesinformers "github.com/openshift/client-go/route/informers/externalversions"
 
+	consolev1client "github.com/openshift/client-go/console/clientset/versioned"
+	// consolev1client "github.com/openshift/client-go/console/clientset/versioned/typed/console/v1"
+	consoleinformers "github.com/openshift/client-go/console/informers/externalversions"
+
 	"github.com/openshift/console-operator/pkg/console/clientwrapper"
+	"github.com/openshift/console-operator/pkg/console/controllers/service"
+	"github.com/openshift/console-operator/pkg/console/metrics"
 	"github.com/openshift/console-operator/pkg/console/operator"
 	"github.com/openshift/library-go/pkg/operator/loglevel"
 )
@@ -61,6 +68,11 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		return err
 	}
 
+	consoleClient, err := consolev1client.NewForConfig(ctx.KubeConfig)
+	if err != nil {
+		return err
+	}
+
 	routesClient, err := routesclient.NewForConfig(ctx.ProtoKubeConfig)
 	if err != nil {
 		return err
@@ -118,6 +130,11 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		oauthinformers.WithTweakListOptions(tweakListOptionsForOAuth),
 	)
 
+	consoleInformers := consoleinformers.NewSharedInformerFactory(
+		consoleClient,
+		resync,
+	)
+
 	operatorClient := &operatorclient.OperatorClient{
 		Informers: operatorConfigInformers,
 		Client:    operatorConfigClient.OperatorV1(),
@@ -129,28 +146,67 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 
 	resourceSyncerInformers, resourceSyncer := getResourceSyncer(ctx, clientwrapper.WithoutSecret(kubeClient), operatorClient)
 
+	consoleMetrics := metrics.Register()
+
 	// TODO: rearrange these into informer,client pairs, NOT separated.
 	consoleOperator := operator.NewConsoleOperator(
-		// informers
-		operatorConfigInformers.Operator().V1().Consoles(), // OperatorConfig
-		configInformers,                                   // ConsoleConfig
-		kubeInformersNamespaced.Core().V1(),               // Secrets, ConfigMaps, Service
-		kubeInformersManagedNamespaced.Core().V1(),        // Managed ConfigMaps
-		kubeInformersNamespaced.Apps().V1().Deployments(), // Deployments
-		routesInformersNamespaced.Route().V1().Routes(),   // Route
-		oauthInformers.Oauth().V1().OAuthClients(),        // OAuth clients
-		// clients
-		operatorConfigClient.OperatorV1(),
+		// top level config
 		configClient.ConfigV1(),
+		configInformers,
+		// operator
+		operatorConfigClient.OperatorV1(),
+		operatorConfigInformers.Operator().V1().Consoles(), // OperatorConfig
 
-		kubeClient.CoreV1(), // Secrets, ConfigMaps, Service
+		// core resources
+		kubeClient.CoreV1(),                 // Secrets, ConfigMaps, Service
+		kubeInformersNamespaced.Core().V1(), // Secrets, ConfigMaps, Service
+		// deployments
 		kubeClient.AppsV1(),
+		kubeInformersNamespaced.Apps().V1().Deployments(), // Deployments
+		// routes
 		routesClient.RouteV1(),
+		routesInformersNamespaced.Route().V1().Routes(), // Route
+		// oauth
 		oauthClient.OauthV1(),
+		oauthInformers.Oauth().V1().OAuthClients(), // OAuth clients
+		// openshift managed
+		kubeInformersManagedNamespaced.Core().V1(), // Managed ConfigMaps
+		// metrics
+		// TODO: when we get to testing, we may want an interface for this
+		consoleMetrics,
+		// event handling
 		versionGetter,
 		recorder,
 		resourceSyncer,
 	)
+	cliDownloadsController := clidownloads.NewCLIDownloadsSyncController(
+		// clients
+		operatorClient,
+		operatorConfigClient.OperatorV1(),
+		consoleClient.ConsoleV1().ConsoleCLIDownloads(),
+		routesClient.RouteV1(),
+
+		// informers
+		operatorConfigInformers.Operator().V1().Consoles(),    // OperatorConfig
+		consoleInformers.Console().V1().ConsoleCLIDownloads(), // ConsoleCliDownloads
+		routesInformersNamespaced.Route().V1().Routes(),       // Routes
+		// recorder
+		recorder,
+	)
+
+	consoleServiceController := service.NewServiceSyncController(
+		// clients
+		operatorConfigClient.OperatorV1().Consoles(), // operator config so we can update status
+		kubeClient.CoreV1(),                          // only needs to interact with the service resource
+		// informers
+		operatorConfigInformers.Operator().V1().Consoles(), // OperatorConfig
+		kubeInformersNamespaced.Core().V1().Services(),     // Services
+		// names
+		api.OpenShiftConsoleNamespace,
+		api.OpenShiftConsoleName,
+		// events
+		recorder,
+	)
 
 	versionRecorder := status.NewVersionGetter()
 	versionRecorder.SetVersion("operator", os.Getenv("RELEASE_VERSION"))
@@ -202,6 +258,7 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		kubeInformersManagedNamespaced,
 		resourceSyncerInformers,
 		operatorConfigInformers,
+		consoleInformers,
 		configInformers,
 		routesInformersNamespaced,
 		oauthInformers,
@@ -209,12 +266,14 @@ func RunOperator(ctx *controllercmd.ControllerContext) error {
 		informer.Start(ctx.Done())
 	}
 
+	go consoleServiceController.Run(1, ctx.Done())
 	go consoleOperator.Run(ctx.Done())
 	go resourceSyncer.Run(1, ctx.Done())
 	go clusterOperatorStatus.Run(1, ctx.Done())
 	go configUpgradeableController.Run(1, ctx.Done())
 	go logLevelController.Run(1, ctx.Done())
 	go managementStateController.Run(1, ctx.Done())
+	go cliDownloadsController.Run(1, ctx.Done())
 	go staleConditionsController.Run(1, ctx.Done())
 
 	<-ctx.Done()
